#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# download-osec-vm-image 
# 
# download the initial OsecVM.ova image
#
# Autor: Oliver Maurhart, <oliver.maurhart@ait.ac.at>
#
# Copyright (C) 2014 AIT Austrian Institute of Technology
# AIT Austrian Institute of Technology GmbH
# Donau-City-Strasse 1 | 1220 Vienna | Austria
# http://www.ait.ac.at
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, 
# Boston, MA  02110-1301, USA.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import os
import sys
import urllib2

from PyQt4 import QtCore
from PyQt4 import QtGui

# local
from environment import Environment


# ------------------------------------------------------------
# code


class DownloadDialog(QtGui.QDialog):
    
    """Download the initial OsecVM image dialog."""

    def __init__(self, parent = None, flags = QtCore.Qt.WindowFlags(0)):
        
        # super call and widget init
        super(DownloadDialog, self).__init__(parent, flags)
        self.setWindowTitle('Download initial OpenSecurity VM image')
        self.setup_ui()

        # setup download thread
        self._downloader = Downloader()
        self._downloader.downloaded.connect(self.downloader_update)
        self._thread = QtCore.QThread()
        self._downloader.moveToThread(self._thread)
        self._downloader.finished.connect(self._thread.quit)
        self._thread.started.connect(self._downloader.work)
        self._thread.finished.connect(self.finished_downloader)
        self._thread.start() 


    def clicked_cancel(self):

        """button cancel clicked"""

        # tell download thread to stop
        self._downloader.stop = True


    def downloader_update(self, current, total):
        
        """New download values present"""
        if self.pbDownload.minimum() != 0:
            self.pbDownload.setMinimum(0)
        if self.pbDownload.maximum() != total:
            self.pbDownload.setMaximum(total)
        self.pbDownload.setValue(current)


    def finished_downloader(self):
        
        """Download thread has finished (either success or abort)"""
        if not self._downloader.error is None:
            QtGui.QMessageBox.critical(None, 'OpenSecurity Download VM image error', self._downloader.error)

        self.accept();


    def setup_ui(self):
        
        """Create the widgets."""
        
        lyMain = QtGui.QVBoxLayout(self)
        lyMain.setContentsMargins(8, 8, 8, 8)

        lbTitle = QtGui.QLabel('Downloading OpenSecurity VM image')
        lyMain.addWidget(lbTitle)
        self.pbDownload = QtGui.QProgressBar()
        self.pbDownload.setTextVisible(True)
        lyMain.addWidget(self.pbDownload)
       
        # buttons
        lyButton = QtGui.QHBoxLayout()
        lyMain.addLayout(lyButton)
        
        lyButton.addStretch(1)
        btnCancel = QtGui.QPushButton('&Cancel', self)
        btnCancel.setMinimumWidth(100)
        lyButton.addWidget(btnCancel)
        
        # connectors
        btnCancel.clicked.connect(self.clicked_cancel)
        
        # reduce to the max
        self.setMinimumWidth(400)
        self.resize(lyMain.minimumSize())


class Downloader(QtCore.QObject):

    """This is the worker thread when downloading"""

    downloaded = QtCore.pyqtSignal(int, int)
    finished = QtCore.pyqtSignal()

    def __init__(self):
        
        # super call 
        super(Downloader, self).__init__()
        self.error = None
        self.stop = False

        # place to store the OsecVM image
        self.vm_path = os.path.join(sys.path[0], '..', 'vm')
        if not os.path.exists(self.vm_path):
            os.mkdir(self.vm_path)
 
       
    def work(self):

        """Thread method: download until finished or stopped"""

        url = "http://service.x-net.at/opensecurity/OsecVM_latest.ova"

        filename_download = os.path.join(self.vm_path, 'OsecVM_latest.ova')
        filename_target = os.path.join(self.vm_path, 'OsecVM.ova')

        # open URL
        try: 
            resp = urllib2.urlopen(url)
        except urllib2.URLError as e:
            self.error = 'URLError: ' + str(e.reason)
            self.finished.emit()
            return
        except urllib2.HTTPError as e:
            self.error = 'HTTPError: ' + str(e.reason)
            self.finished.emit()
            return

        # download a CHUNK of data and then update the GUI
        totalSize = int(resp.info().getheader('Content-Length').strip())
        currentSize = 0
        CHUNK_SIZE = 32768
        self.downloaded.emit(currentSize, totalSize)

        f = open(filename_download, 'w')
        while (not self.stop) and (currentSize < totalSize):
            data = resp.read(CHUNK_SIZE)
            currentSize += len(data)
            f.write(data)
            self.downloaded.emit(currentSize, totalSize)

        f.flush()
        f.close()

        # rename the download to OsecVM.ova
        if os.path.exists(filename_target):
            os.remove(filename_target)
        os.rename(filename_download, filename_target)

        self.finished.emit()


def main():

    """entry point"""
    
    # launch Qt
    global app
    app = QtGui.QApplication(sys.argv)

    # prebuild the pixmap cache: fetch all jpg, png and jpeg images and load them
    image_path = os.path.join(sys.path[0], '..', 'gfx')
    for file in os.listdir(image_path):
        if file.lower().rpartition('.')[2] in ('jpg', 'png', 'jpeg'):
            QtGui.QPixmapCache.insert(file.lower().rpartition('.')[0], QtGui.QPixmap(os.path.join(image_path, file)))
            
    # we should have now our application icon
    app.setWindowIcon(QtGui.QIcon(QtGui.QPixmapCache.find('opensecurity_icon_64')))

    w = DownloadDialog()
    w.show()

    # on with the show
    sys.exit(app.exec_())
   
    
# start
if __name__ == "__main__":
    main()

